/* Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#ifndef PRUSS_REGISTER_H_
#define PRUSS_REGISTER_H_


#define PRUSS_BASE		0x54400000

#define	PRUSS_INTC			0x20000
#define PRUSS0_INTC			0x60000
#define	PRUSS_PRU0_CTRL		0x22000
#define	PRUSS_PRU1_CTRL		0x24000
#define PRUSS_CFG			0x26000
#define PRUSS_MII_RT_CFG	0x32000
#define PRUSS_MDIO			0x32400
#define PRUSS_MDIOCNTROL	0x04
#define PRUSS_MDIOUSERACCESS0 0x80
#define PRUSS_MDIOUSERACCESS1 0x88
#define PRUSS_MDIO_USERPHYSEL0	0x84
#define PRUSS_MDIO_USERINTMASKSET	0x28

/*
 * INTC registers
 */
#define INTC_SECR0		0x280
#define INTC_SECR1		0x284


/*
 * PRU Control
 */
#define PRUSS_PRU0_CTRL_CONTROL		(PRUSS_PRU0_CTRL + 0x00)
#define PRUSS_PRU0_CTRL_STATUS		(PRUSS_PRU0_CTRL + 0x04)
#define PRUSS_PRU0_CTRL_WAKEUP_EN	(PRUSS_PRU0_CTRL + 0x08)
#define PRUSS_PRU0_CTRL_CYCLE		(PRUSS_PRU0_CTRL + 0x0C)
#define PRUSS_PRU0_CTRL_STALL		(PRUSS_PRU0_CTRL + 0x10)
#define PRUSS_PRU0_CTRL_CTBIR0		(PRUSS_PRU0_CTRL + 0x20)
#define PRUSS_PRU0_CTRL_CTBIR1		(PRUSS_PRU0_CTRL + 0x24)
#define PRUSS_PRU0_CTRL_CTPPR0		(PRUSS_PRU0_CTRL + 0x28)
#define PRUSS_PRU0_CTRL_CTPPR1		(PRUSS_PRU0_CTRL + 0x2C)

#define PRUSS_PRU1_CTRL_CONTROL		(PRUSS_PRU1_CTRL + 0x00)
#define PRUSS_PRU1_CTRL_STATUS		(PRUSS_PRU1_CTRL + 0x04)
#define PRUSS_PRU1_CTRL_WAKEUP_EN	(PRUSS_PRU1_CTRL + 0x08)
#define PRUSS_PRU1_CTRL_CYCLE		(PRUSS_PRU1_CTRL + 0x0C)
#define PRUSS_PRU1_CTRL_STALL		(PRUSS_PRU1_CTRL + 0x10)
#define PRUSS_PRU1_CTRL_CTBIR0		(PRUSS_PRU1_CTRL + 0x20)
#define PRUSS_PRU1_CTRL_CTBIR1		(PRUSS_PRU1_CTRL + 0x24)
#define PRUSS_PRU1_CTRL_CTPPR0		(PRUSS_PRU1_CTRL + 0x28)
#define PRUSS_PRU1_CTRL_CTPPR1		(PRUSS_PRU1_CTRL + 0x2C)


/*
 * CFG Register
 */
#define PRUSS_CFG_REVID		(PRUSS_CFG + 0x00)
#define PRUSS_CFG_SYSCFG	(PRUSS_CFG + 0x04)
#define PRUSS_CFG_GPCFG0	(PRUSS_CFG + 0x08)
#define PRUSS_CFG_GPCFG1	(PRUSS_CFG + 0x0C)
#define PRUSS_CFG_CGR		(PRUSS_CFG + 0x10)
#define PRUSS_CFG_ISRP		(PRUSS_CFG + 0x14)
#define PRUSS_CFG_ISP		(PRUSS_CFG + 0x18)
#define PRUSS_CFG_IESP		(PRUSS_CFG + 0x1C)
#define PRUSS_CFG_IECP		(PRUSS_CFG + 0x20)
#define PRUSS_CFG_SCRP		(PRUSS_CFG + 0x24)
#define PRUSS_CFG_PMAO		(PRUSS_CFG + 0x28)
#define PRUSS_CFG_MII_RT	(PRUSS_CFG + 0x2C)
#define PRUSS_CFG_IEPCLK	(PRUSS_CFG + 0x30)
#define PRUSS_CFG_SPP		(PRUSS_CFG + 0x34)
#define PRUSS_CFG_PIN_MX	(PRUSS_CFG + 0x40)

/*
 * MII_RT_CFG
 */
#define PRUSS_MII_RT_CFG_RXCFG0		(PRUSS_MII_RT_CFG + 0x00)
#define PRUSS_MII_RT_CFG_RXCFG1		(PRUSS_MII_RT_CFG + 0x04)
#define PRUSS_MII_RT_CFG_TXCFG0		(PRUSS_MII_RT_CFG + 0x10)
#define PRUSS_MII_RT_CFG_TXCFG1		(PRUSS_MII_RT_CFG + 0x14)
#define PRUSS_MII_RT_CFG_TXCRC0		(PRUSS_MII_RT_CFG + 0x20)
#define PRUSS_MII_RT_CFG_TXCRC1		(PRUSS_MII_RT_CFG + 0x24)
#define PRUSS_MII_RT_CFG_TXIPG0		(PRUSS_MII_RT_CFG + 0x30)
#define PRUSS_MII_RT_CFG_TXIPG1		(PRUSS_MII_RT_CFG + 0x34)
#define PRUSS_MII_RT_CFG_PRS0		(PRUSS_MII_RT_CFG + 0x38)
#define PRUSS_MII_RT_CFG_PRS1		(PRUSS_MII_RT_CFG + 0x3C)
#define PRUSS_MII_RT_CFG_RXFRMS0	(PRUSS_MII_RT_CFG + 0x40)
#define PRUSS_MII_RT_CFG_RXFRMS1	(PRUSS_MII_RT_CFG + 0x44)
#define PRUSS_MII_RT_CFG_RXPCNT0	(PRUSS_MII_RT_CFG + 0x48)
#define PRUSS_MII_RT_CFG_RXPCNT1	(PRUSS_MII_RT_CFG + 0x4C)
#define PRUSS_MII_RT_CFG_RXERR0		(PRUSS_MII_RT_CFG + 0x50)
#define PRUSS_MII_RT_CFG_RXERR1		(PRUSS_MII_RT_CFG + 0x54)


/* SYSCFG */
#define PRUSS_SYSCFG_SUB_MWAIT_READY	0
#define PRUSS_SYSCFG_SUB_MWAIT_WAIT		1
#define PRUSS_SYSCFG_SUB_MWAIT_SHIFT	5

#define PRUSS_SYSCFG_STANDBY_INIT_STANDBY	1
#define PRUSS_SYSCFG_STANDBY_INIT_ENABLE	0
#define PRUSS_SYSCFG_STANDBY_INIT_SHIFT		4

#define PRUSS_SYSCFG_STANDBY_MODE_FORCE		0
#define PRUSS_SYSCFG_STANDBY_MODE_NO		1
#define PRUSS_SYSCFG_STANDBY_MODE_SMART		2
#define PRUSS_SYSCFG_STANDBY_MODE_SHIFT		2

#define PRUSS_SYSCFG_IDLE_MODE_FORCE		0
#define PRUSS_SYSCFG_IDLE_MODE_NO			1
#define PRUSS_SYSCFG_IDLE_MODE_SMART		2
#define PRUSS_SYSCFG_IDLE_MODE_SHIFT		0

/* GPCFG0 */
#define PRUSS_GPCFG0_PRU0_GPO_SH_SEL_SH0	0
#define PRUSS_GPCFG0_PRU0_GPO_SH_SEL_SH1	1
#define PRUSS_GPCFG0_PRU0_GPO_SH_SEL_SHIFT	25

#define PRUSS_GPCFG0_PRU0_DIV1_SHIFT		20

#define PRUSS_GPCFG0_PRU0_DIV0_SHIFT		15

#define PRUSS_GPCFG0_PRU0_GPO_MODE_DIRECT	0
#define PRUSS_GPCFG0_PRU0_GPO_MODE_SERIAL	1
#define PRUSS_GPCFG0_PRU0_GPO_MODE_SHIFT	14

#define PRUSS_GPCFG0_PRU0_GPI_SB_SHIFT		13

#define PRUSS_GPCFG0_PRU0_GPI_DIV1_SHIFT	8

#define PRUSS_GPCFG0_PRU0_GPI_DIV0_SHIFT	3

#define PRUSS_GPCFG0_PRU0_GPI_CLK_MODE_POSITIVE		0
#define PRUSS_GPCFG0_PRU0_GPI_CLK_MODE_NEGATIVE		1
#define PRUSS_GPCFG0_PRU0_GPI_CLK_MODE_SHIFT		2

#define PRUSS_GPCFG0_PRU0_GPI_MODE_DIRECTMODE		0
#define PRUSS_GPCFG0_PRU0_GPI_MODE_PARALLELMODE 	1
#define PRUSS_GPCFG0_PRU0_GPI_MODE_SHIFTMODE 		2
#define PRUSS_GPCFG0_PRU0_GPI_MODE_MIIMODE			3
#define PRUSS_GPCFG0_PRU0_GPI_MODE_SHIFT			0

/* GPCFG1 */
#define PRUSS_GPCFG1_PRU1_GPO_SH_SEL_SH0	0
#define PRUSS_GPCFG1_PRU1_GPO_SH_SEL_SH1	1
#define PRUSS_GPCFG1_PRU1_GPO_SH_SEL_SHIFT	25

#define PRUSS_GPCFG1_PRU1_DIV1_SHIFT		20

#define PRUSS_GPCFG1_PRU1_DIV0_SHIFT		15

#define PRUSS_GPCFG1_PRU1_GPO_MODE_DIRECT	0
#define PRUSS_GPCFG1_PRU1_GPO_MODE_SERIAL	1
#define PRUSS_GPCFG1_PRU1_GPO_MODE_SHIFT	14

#define PRUSS_GPCFG1_PRU1_GPI_SB_SHIFT		13

#define PRUSS_GPCFG1_PRU1_GPI_DIV1_SHIFT	8

#define PRUSS_GPCFG1_PRU1_GPI_DIV0_SHIFT	3

#define PRUSS_GPCFG1_PRU1_GPI_CLK_MODE_POSITIVE		0
#define PRUSS_GPCFG1_PRU1_GPI_CLK_MODE_NEGATIVE		1
#define PRUSS_GPCFG1_PRU1_GPI_CLK_MODE_SHIFT		2

#define PRUSS_GPCFG1_PRU1_GPI_MODE_DIRECTMODE		0
#define PRUSS_GPCFG1_PRU1_GPI_MODE_PARALLELMODE 	1
#define PRUSS_GPCFG1_PRU1_GPI_MODE_SHIFTMODE 		2
#define PRUSS_GPCFG1_PRU1_GPI_MODE_MIIMODE			3
#define PRUSS_GPCFG1_PRU1_GPI_MODE_SHIFT			0

/* CGR */
#define PRUSS_CGR_IEP_CLK_EN_DISABLE	0
#define PRUSS_CGR_IEP_CLK_EN_ENABLE		1
#define PRUSS_CGR_IEP_CLK_EN_SHIFT		17

#define PRUSS_CGR_IEP_CLK_STOP_ACK_NOTREADY		0
#define PRUSS_CGR_IEP_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_IEP_CLK_STOP_ACK_SHIFT		16

#define PRUSS_CGR_IEP_CLK_STOP_REQ_NOTREQUEST	0
#define PRUSS_CGR_IEP_CLK_STOP_REQ_REQUEST		1
#define PRUSS_CGR_IEP_CLK_STOP_REQ_SHIFT		15

#define PRUSS_CGR_ECAP_CLK_EN_DISABLE	0
#define PRUSS_CGR_ECAP_CLK_EN_ENABLE	1
#define PRUSS_CGR_ECAP_CLK_EN_SHIFT		14

#define PRUSS_CGR_ECAP_CLK_STOP_ACK_NOTREADY	0
#define PRUSS_CGR_ECAP_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_ECAP_CLK_STOP_ACK_SHIFT		13

#define PRUSS_CGR_ECAP_CLK_STOP_REQ_NOTREADY	0
#define PRUSS_CGR_ECAP_CLK_STOP_REQ_READY		1
#define PRUSS_CGR_ECAP_CLK_STOP_REQ_SHIFT		12

#define PRUSS_CGR_UART_CLK_EN_DISABLE	0
#define PRUSS_CGR_UART_CLK_EN_ENABLE	1
#define PRUSS_CGR_UART_CLK_EN_SHIFT		11

#define PRUSS_CGR_UART_CLK_STOP_ACK_NOTREADY	0
#define PRUSS_CGR_UART_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_UART_CLK_STOP_ACK_SHIFT		10

#define PRUSS_CGR_UART_CLK_STOP_REQ_NOTREADY	0
#define PRUSS_CGR_UART_CLK_STOP_REQ_READY		1
#define PRUSS_CGR_UART_CLK_STOP_REQ_SHIFT		9

#define PRUSS_CGR_INTC_CLK_EN_DISABLE	0
#define PRUSS_CGR_INTC_CLK_EN_ENABLE	1
#define PRUSS_CGR_INTC_CLK_EN_SHIFT		8

#define PRUSS_CGR_INTC_CLK_STOP_ACK_NOTREADY	0
#define PRUSS_CGR_INTC_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_INTC_CLK_STOP_ACK_SHIFT		7

#define PRUSS_CGR_INTC_CLK_STOP_REQ_NOTREADY	0
#define PRUSS_CGR_INTC_CLK_STOP_REQ_READY		1
#define PRUSS_CGR_INTC_CLK_STOP_REQ_SHIFT		6

#define PRUSS_CGR_PRU1_CLK_EN_DISABLE	0
#define PRUSS_CGR_PRU1_CLK_EN_ENABLE	1
#define PRUSS_CGR_PRU1_CLK_EN_SHIFT		5

#define PRUSS_CGR_PRU1_CLK_STOP_ACK_NOTREADY	0
#define PRUSS_CGR_PRU1_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_PRU1_CLK_STOP_ACK_SHIFT		4

#define PRUSS_CGR_PRU1_CLK_STOP_REQ_NOTREADY	0
#define PRUSS_CGR_PRU1_CLK_STOP_REQ_READY		1
#define PRUSS_CGR_PRU1_CLK_STOP_REQ_SHIFT		3

#define PRUSS_CGR_PRU0_CLK_EN_DISABLE	0
#define PRUSS_CGR_PRU0_CLK_EN_ENABLE	1
#define PRUSS_CGR_PRU0_CLK_EN_SHIFT		2

#define PRUSS_CGR_PRU0_CLK_STOP_ACK_NOTREADY	0
#define PRUSS_CGR_PRU0_CLK_STOP_ACK_READY		1
#define PRUSS_CGR_PRU0_CLK_STOP_ACK_SHIFT		1

#define PRUSS_CGR_PRU0_CLK_STOP_REQ_NOTREADY	0
#define PRUSS_CGR_PRU0_CLK_STOP_REQ_READY		1
#define PRUSS_CGR_PRU0_CLK_STOP_REQ_SHIFT		0

/* ISRP */
#define PRUSS_ISRP_RAM_PE_RAW_SHIFT			16
#define PRUSS_ISRP_PRU1_DMEM_PE_RAW_SHIFT	12
#define PRUSS_ISRP_PRU1_IMEM_PE_RAW_SHIFT	8
#define PRUSS_ISRP_PRU0_DMAM_PE_RAW_SHIFT	4
#define PRUSS_ISRP_PRU0_IMEM_PE_RAW_SHIFT	0

/* ISP */
#define PRUSS_ISP_RAM_PE_SHIFT			16
#define PRUSS_ISP_PRU1_DRAM_PE_SHIFT	12
#define PRUSS_ISP_PRU1_IMEM_PE_SHIFT	8
#define PRUSS_ISP_PRU0_DMEM_PE_SHIFT	4
#define PRUSS_ISP_PRU0_IMEM_PE_SHIFT	0

/* IESP */
#define PRUSS_IESP_RAM_PE_SET_SHIFT			16
#define PRUSS_IESP_PRU1_DRAM_PE_SET_SHIFT	12
#define PRUSS_IESP_PRU1_IMEM_PE_SET_SHIFT	8
#define PRUSS_IESP_PRU0_DMEM_PE_SET_SHIFT	4
#define PRUSS_IESP_PRU0_IMEM_PE_SET_SHIFT	0

/* IECP */
#define PRUSS_IECP_RAM_PE_CLR_SHIFT			16
#define PRUSS_IECP_PRU1_DRAM_PE_CLR_SHIFT	12
#define PRUSS_IECP_PRU1_IMEM_PE_CLR_SHIFT	8
#define PRUSS_IECP_PRU0_DMEM_PE_CLR_SHIFT	4
#define PRUSS_IECP_PRU0_IMEM_PE_CLR_SHIFT	0

/* SCRP */
#define PRUSS_SCRP_18_SHIFT		21
#define PRUSS_SCRP_17_SHIFT		20
#define PRUSS_SCRP_16_SHIFT		19
#define PRUSS_SCRP_15_SHIFT		18
#define PRUSS_SCRP_14_SHIFT		17
#define PRUSS_SCRP_13_SHIFT		16
#define PRUSS_SCRP_12_SHIFT		15
#define PRUSS_SCRP_11_SHIFT		14
#define PRUSS_SCRP_10_SHIFT		13
#define PRUSS_SCRP_9_SHIFT		12
#define PRUSS_SCRP_8_SHIFT		11
#define PRUSS_SCRP_7_SHIFT		10
#define PRUSS_SCRP_6_SHIFT		9
#define PRUSS_SCRP_5_SHIFT		8
#define PRUSS_SCRP_4_SHIFT		6
#define PRUSS_SCRP_3_SHIFT		4
#define PRUSS_SCRP_2_SHIFT		2
#define PRUSS_SCRP_1_SHIFT		0

/* PMAO */
#define PRUSS_PMAO_PRU1_DISABLE	0
#define PRUSS_PMAO_PRU1_ENABLE		1
#define PRUSS_PMAO_PRU1_SHIFT		1

#define PRUSS_PMAO_PRU0_DISABLE	0
#define PRUSS_PMAO_PRU0_ENABLE		1
#define PRUSS_PMAO_PRU0_SHIFT		0

/* MII_RT */
#define PRUSS_MII_RT_EVENT_EN_DISABLE	0
#define PRUSS_MII_RT_EVENT_EN_ENABLE	1
#define PRUSS_MII_RT_EVENT_EN_SHIFT		0

/* IEPCLK */
#define PRUSS_IEPCLK_OCP_EN_IEPCLK		0
#define PRUSS_IEPCLK_OCP_EN_OCPCLK		1
#define PRUSS_IEPCLK_OCP_EN_SHIFT		0

/* SPP */
#define PRUSS_SPP_XFER_SHIFT_EN_DISABLE		0
#define PRUSS_SPP_XFER_SHIFT_EN_ENABLE		1
#define PRUSS_SPP_XFER_SHIFT_EN_SHIFT		1

#define PRUSS_SPP_PRU1_PAD_HP_EN_PRU0		0
#define PRUSS_SPP_PRU1_PAD_HP_EN_PRU1		1
#define PRUSS_SPP_PRU1_PAD_HP_EN_SHIFT		0

/* PIN_MX */
#define PRUSS_PIN_MUX_SEL0_SHIFT	0
#define PRUSS_PIN_MUX_SEL1_SHIFT	1


/* RXCFG0 */
#define PRUSS_RXCFG0_RX_AUTO_FWD_PRE_DISABLE	0
#define PRUSS_RXCFG0_RX_AUTO_FWD_PRE_ENABLE		1
#define PRUSS_RXCFG0_RX_AUTO_FWD_PRE_SHIFT		6

#define PRUSS_RXCFG0_RX_BYTE_SWAP_DISABLE		0
#define PRUSS_RXCFG0_RX_BYTE_SWAP_ENABLE		1
#define PRUSS_RXCFG0_RX_BYTE_SWAP_SHIFT			5

#define PRUSS_RXCFG0_RX_L2_DISABLE				0
#define PRUSS_RXCFG0_RX_L2_ENABLE				1
#define PRUSS_RXCFG0_RX_L2_SHIFT				4

#define PRUSS_RXCFG0_RX_MUX_SELECT_PORT0		0
#define PRUSS_RXCFG0_RX_MUX_SELECT_PORT1		1
#define PRUSS_RXCFG0_RX_MUX_SELECT_SHIFT		3

#define PRUSS_RXCFG0_RX_CUT_PREAMBLE_DISABLE	0
#define PRUSS_RXCFG0_RX_CUT_PREAMBLE_ENABLE		1
#define PRUSS_RXCFG0_RX_CUT_PREAMBLE_SHIFT		2

#define PRUSS_RXCFG0_RX_DISABLE			0
#define PRUSS_RXCFG0_RX_ENABLE			1
#define PRUSS_RXCFG0_RX_ENABLE_SHIFT			0

/* RXCFG1 */
#define PRUSS_RXCFG1_RX_AUTO_FWD_PRE_DISABLE	0
#define PRUSS_RXCFG1_RX_AUTO_FWD_PRE_ENABLE		1
#define PRUSS_RXCFG1_RX_AUTO_FWD_PRE_SHIFT		6

#define PRUSS_RXCFG1_RX_BYTE_SWAP_DISABLE		0
#define PRUSS_RXCFG1_RX_BYTE_SWAP_ENABLE		1
#define PRUSS_RXCFG1_RX_BYTE_SWAP_SHIFT			5

#define PRUSS_RXCFG1_RX_L2_DISABLE				0
#define PRUSS_RXCFG1_RX_L2_ENABLE				1
#define PRUSS_RXCFG1_RX_L2_SHIFT				4

#define PRUSS_RXCFG1_RX_MUX_SELECT_PORT0		0
#define PRUSS_RXCFG1_RX_MUX_SELECT_PORT1		1
#define PRUSS_RXCFG1_RX_MUX_SELECT_SHIFT		3

#define PRUSS_RXCFG1_RX_CUT_PREAMBLE_DISABLE	0
#define PRUSS_RXCFG1_RX_CUT_PREAMBLE_ENABLE		1
#define PRUSS_RXCFG1_RX_CUT_PREAMBLE_SHIFT		2

#define PRUSS_RXCFG1_RX_DISABLE			0
#define PRUSS_RXCFG1_RX_ENABLE			1
#define PRUSS_RXCFG1_RX_ENABLE_SHIFT			0


/* TXCFG0 */
#define PRUSS_TXCFG0_TX_CLK_DELAY_SHIFT			28

#define PRUSS_TXCFG0_TX_START_DELAY_SHIFT		16

#define PRUSS_TXCFG0_TX_AUTO_ESC_ERR_DISABLE	0
#define PRUSS_TXCFG0_TX_AUTO_ESC_ERR_ENABLE		1
#define PRUSS_TXCFG0_TX_AUTO_ESC_ERR_SHIFT		10

#define PRUSS_TXCFG0_TX_AUTO_SEQUENCE_DISABLE	0
#define PRUSS_TXCFG0_TX_AUTO_SEQUENCE_ENABLE	1
#define PRUSS_TXCFG0_TX_AUTO_SEQUENCE_SHIFT		9

#define PRUSS_TXCFG0_TX_MUX_SEL_PORT0			0
#define PRUSS_TXCFG0_TX_MUX_SEL_PORT1			1
#define PRUSS_TXCFG0_TX_MUX_SEL_SHIFT			8

#define PRUSS_TXCFG0_TX_BYTE_SWAP_DISABLE		0
#define PRUSS_TXCFG0_TX_BYTE_SWAP_ENABLE		1
#define PRUSS_TXCFG0_TX_BYTE_SWAP_SHIFT			3

#define PRUSS_TXCFG0_TX_EN_MODE_DISABLE			0
#define PRUSS_TXCFG0_TX_EN_MODE_ENABLE			1
#define PRUSS_TXCFG0_TX_EN_MODE_SHIFT			2

#define PRUSS_TXCFG0_TX_AUTO_PREAMBLE_DISABLE	0
#define PRUSS_TXCFG0_TX_AUTO_PREAMBLE_ENABLE	1
#define PRUSS_TXCFG0_TX_AUTO_PREAMBLE_SHIFT		1

#define PRUSS_TXCFG0_TX_DISABLE			0
#define PRUSS_TXCFG0_TX_ENABLE			1
#define PRUSS_TXCFG0_TX_ENABLE_SHIFT 	0

/* TXCFG1 */
#define PRUSS_TXCFG1_TX_CLK_DELAY_SHIFT			28

#define PRUSS_TXCFG1_TX_START_DELAY_SHIFT		16

#define PRUSS_TXCFG1_TX_AUTO_ESC_ERR_DISABLE	0
#define PRUSS_TXCFG1_TX_AUTO_ESC_ERR_ENABLE		1
#define PRUSS_TXCFG1_TX_AUTO_ESC_ERR_SHIFT		10

#define PRUSS_TXCFG1_TX_AUTO_SEQUENCE_DISABLE	0
#define PRUSS_TXCFG1_TX_AUTO_SEQUENCE_ENABLE	1
#define PRUSS_TXCFG1_TX_AUTO_SEQUENCE_SHIFT		9

#define PRUSS_TXCFG1_TX_MUX_SEL_PORT0			0
#define PRUSS_TXCFG1_TX_MUX_SEL_PORT1			1
#define PRUSS_TXCFG1_TX_MUX_SEL_SHIFT			8

#define PRUSS_TXCFG1_TX_BYTE_SWAP_DISABLE		0
#define PRUSS_TXCFG1_TX_BYTE_SWAP_ENABLE		1
#define PRUSS_TXCFG1_TX_BYTE_SWAP_SHIFT			3

#define PRUSS_TXCFG1_TX_EN_MODE_DISABLE			0
#define PRUSS_TXCFG1_TX_EN_MODE_ENABLE			1
#define PRUSS_TXCFG1_TX_EN_MODE_SHIFT			2

#define PRUSS_TXCFG1_TX_AUTO_PREAMBLE_DISABLE	0
#define PRUSS_TXCFG1_TX_AUTO_PREAMBLE_ENABLE	1
#define PRUSS_TXCFG1_TX_AUTO_PREAMBLE_SHIFT		1

#define PRUSS_TXCFG1_TX_DISABLE			0
#define PRUSS_TXCFG1_TX_ENABLE			1
#define PRUSS_TXCFG1_TX_ENABLE_SHIFT 	0

/* RXFRMS0 */
#define PRUSS_RXFRMS0_RX_MAX_FRM_CNT_SHIFT		16

#define PRUSS_RXFRMS0_RX_MIN_FRM_CNT_SHIFT		0

/* RXFRMS1 */
#define PRUSS_RXFRMS1_RX_MAX_FRM_CNT_SHIFT		16

#define PRUSS_RXFRMS1_RX_MIN_FRM_CNT_SHIFT		0

/* RXPCNT0 */
#define PRUSS_RXPCNT0_RX_MAX_PRE_CNT_SHIFT				4

#define PRUSS_RXPCNT0_RX_MIN_PRE_CNT_SHIFT				0

/* RXPCNT1 */
#define PRUSS_RXPCNT1_RX_MAX_PRE_CNT_SHIFT				4

#define PRUSS_RX_RXPCNT1_MIN_PRE_CNT_SHIFT				0

/* RXERR0 */
#define PRUSS_RXERR0_RX_MAX_FRM_CNT_ERR		0x08
#define PRUSS_RXERR0_RX_MIN_FRM_CNT_ERR		0x04
#define PRUSS_RXERR0_RX_MAX_PRE_CNT_ERR		0x02
#define PRUSS_RXERR0_RX_MIN_PRE_CNT_ERR		0x01

#endif /* PRUSS_REGISTER_H_ */
