;
;Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
;
;
;  Redistribution and use in source and binary forms, with or without
;  modification, are permitted provided that the following conditions
;  are met:
;
;    Redistributions of source code must retain the above copyright
;    notice, this list of conditions and the following disclaimer.
;
;    Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the
;    distribution.
;
;    Neither the name of Texas Instruments Incorporated nor the names of
;    its contributors may be used to endorse or promote products derived
;    from this software without specific prior written permission.
;
;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
;  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
;  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
;  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; 	file:   i2c_Smbus_fw.asm
;	                              /|\  /|\
;                 AM437x_IDK     1.0k  1.0k    TUSB8041
;                     Master	   |    |     	Slave
;             |-----------------|  |    | |-----------------|
;             |             SCL |--|----+>| SCL      		|
;             |                 |  |      |                 |
;             |             SDA |<-+----->| SDA       		|
;             |                 |         |                 |


;
; 	brief: I2C/SMBus communication interface between PRU and TUSB8041 USB Hub
;
;	Version		Description						Author
; 	0.2      	I2C - PRU communication	    	Muhammad Haisem Khan

;AM437x PRU			TUSB8041
;SCL (R30.t13)		SCL
;SDA (R30.t11)		SDA

SDA		.SET	11									; pin 17 on J16 as GPO11
SCL		.SET	13									; pin 19 on J16 as GPO13

;************************************************************************************
; 							Registers Name Assignment Section
;************************************************************************************
	.asg 	R7.b0, tx_data							;Used for storing 8 bits that are transmitted towards slave
	.asg 	R7.b1, bit_count						;Keeps track of how many bits from a byte has been sent
	.asg	R7.b2, byte_count						;Keeps track of bytes that has been sent in SMBus Write_Word & Write_Block Commands
	.asg	R7.b3, total_byte_count_N				;Contains total number of bytes that needs to be sent in SMBus Write_Block Command

	.asg	R23, status_value_IRQSTS_RAW			;Contains status value for ARDY and NACK bit from IRQSTS_RAW register
	.asg	R24, status_value_IRQEN_SET				;Contains status value for ARDY and NACK bit from IRQEN_SET register
	.asg	R25, status_value						;Contains status value for ARDY and NACK bit added from IRQSTS_RAW and IRQEN_SET register

	.asg	R3.w0, delay_count_reg_quater_cycle		;Contains delay count value for quater clock cycle
	.asg	R3.w2, delay_count_reg_half_cycle	 	;Contains delay count value for half clock cycle

	;.asg	R4.b0, SMBus_frequency_sel				;Used for storing the selected frequency
	.asg	R4.b1, pnmx_val							;Used for attaining the current pinmux value of SDA line (Output = 4, Input = 5)
	.asg	R4.b2, SMBus_protocol_sel_reg			;Used for storing the current SMBus Protocol Command that needs to be executed
	.asg	R4.b3, slave_addr						;Contains SMBus Slave Address in 7 MSB bits (TUSB8041 = 1001000) and R/W LSB bit (R=1, W=0)

	.asg	R8.b0, rx_data							;Used for storing 8 bits that are receieved from slave
	.asg 	R8.b2, rx_total_byte_count_N			;Contains total number of bytes that are to be sent by slave in SMBus Read_Block Command

	.asg	R5, temp_reg							;Used in Clearing INTC for Pinmux change via EDMA and interrupt from ARM2PRU

	.asg	R10.b0, rd_wr_bit        				;Contains Information about Read or Write direction bit in Quick Command
	.asg	R10.b2, last_bit_sent					;Contains the value '0' or '1' of the last sent bit
	.asg	R10.b3, bit_count_and_last_bit_sent		;Contains the sum of the bit_count and last_bit_sent, used for earlier EDMA triggering logic when last bit is '1'

	.asg	R6, pru1_intc							;Points to address of PRU1_ICSS1

	.asg	R13, mem_idx_tx_data					;Points to PRU Memory area where TX data is stored

	.asg	R14, mem_idx_rx_data					;Points to PRU Memory area where RX data is stored

	.asg	R9, sda_pin_reg							;Points to address of Control Module (44E10000) + Offset for CTRL_CONF_CAM1_DATA3 (9F4) the pin used as SDA

	.asg	R11.w0, ack_reg							;Contains Ack bit information from slave

	.asg	R29.w0, RET_ADDR0						;Contains return address for Function Call 2 (M_CALL)
	.asg	R28.w0, RET_ADDR1						;Contains return address for Function Call 2 (M_CALL1)
	.asg	R27.w0, RET_ADDR2						;Contains return address for Function Call 2 (M_CALL2)

;************************************************************************************
; 									Macro Section
;************************************************************************************
;Macro for inserting NOP
M_NOP .macro
	 ADD 	R0, R0, 0
	.endm

;Macros for returning from a function call
M_RET .macro
	JMP		RET_ADDR0
	.endm
M_RET1 .macro
	JMP		RET_ADDR1
	.endm

;Macros for a function call
M_CALL .macro func
	jal		RET_ADDR0, func
	.endm
M_CALL1 .macro func
	jal		RET_ADDR1, func
	.endm

;Macro for copying 32bits from source to destination register
M_MOV32 .macro dst, src
	LDI		dst.w0, src & 0xFFFF
	LDI		dst.w2, src >> 16
	.endm

;Macro for clearing System Event Status for INTC-ICSS0 and INTC-ICSS1
M_CLR_INTC_PINMUX .macro
	; clear event 16
	LDI		temp_reg, CLEAR_EV_16_VAL
	SBCO	&temp_reg, CONST_ICSS_INTC, ICSS_INTC_SICR, 4
	; clear event 56
	LDI		temp_reg, CLEAR_EV_56_VAL
	SBBO	&temp_reg, pru1_intc, ICSS_INTC_SICR, 4
	.endm

;Macro for clearing System Event 18 for Interrupt from ARM to PRU
M_CLR_INTC_ARM2PRU .macro
	; clear event 18
	LDI		temp_reg, CLEAR_EV_18_VAL
	SBCO	&temp_reg, CONST_ICSS_INTC, ICSS_INTC_SICR, 4
	.endm

;************************************************************************************
; 								Defines Section
;************************************************************************************
WRITE_BIT						.SET	0x0			;Write bit value
READ_BIT						.SET	0x1			;Read bit value
READ_WRITE_BIT					.SET	9			;Positon of Rd/Wr bit -- 9th bit in I2C_CON is TRX which is used as Rd/Wr direction
ARDY_BIT						.SET	2			;Position of ARDY bit
NACK_BIT						.SET	1			;Position of NACK bit
PRU_ICSS0_EVENT_BIT				.SET	30			;Bit position for ICSS0 EVENT in R31

;---------------------------------------------------------
; 					SMBus Commands
;---------------------------------------------------------
NO_COMMAND						.SET	0x0			;Value if No Command Selected
QUICK_COMMAND					.SET	0x1			;Value if Quick Command Selected
SEND_BYTE						.SET	0x2			;Value if Send_Byte Command Selected
RECEIVE_BYTE					.SET	0x3			;Value if Receive_Byte Command Selected
WRITE_BYTE						.SET	0x4			;Value if Write_Byte Command Selected
WRITE_WORD						.SET	0x5			;Value if Write_Word Command Selected
READ_BYTE						.SET	0x6			;Value if Read_Byte Command Selected
READ_WORD						.SET	0x7			;Value if Read_Word Command Selected
WRITE_BLK						.SET	0x8			;Value if Write_Block Command Selected
READ_BLK						.SET	0x9			;Value if Read_Block Command Selected

;---------------------------------------------------------
;	Names of Memory Location Offsets in PRU Local Memory
;---------------------------------------------------------
I2C_IRQSTS_RAW					.SET	0x24		;I2C Status Raw Register / bit ARDY = 2, bit NACK = 1
I2C_IRQSTS						.SET	0x28		;I2C Status Register / bit ARDY = 2, bit NACK = 1
I2C_IRQEN_SET					.SET	0x2C		;I2C Interrupt Enable Set Register / bit ARDY = 2, bit NACK = 1
I2C_CNT							.SET	0x98		;Data Count Register
I2C_CON							.SET	0xA4		;I2C Configuration Register --- 9th bit TRX used as Rd/Wr direction
I2C_SA							.SET	0xAC		;I2C Slave Address Register
SDA_PINMMUX_VALUE_OUTPUT		.SET	0xE0		;SDA pinmux value for onput
SDA_PINMMUX_VALUE_INPUT			.SET	0xE4		;SDA pinmux value for iutput
CLOCK_HALF_CYCLE_DELAY_VALUE	.SET	0xE8		;Contains delay value for half clock cycle
COMMAND_CODE					.SET	0xEC		;Contains Command Code Information
SELECTED_SMBUS_COMMAND			.SET	0xF0		;Contains Information that which SMBus Command is selected
TX_DATA_LOCATION				.SET 	0x100		;Location where TX data is located
RX_DATA_LOCATION				.SET	0x200		;Location where RX data is stored

;---------------------------------------------------------
; 			Pinmuxing Logic and INTC Defines
;---------------------------------------------------------

;From Constant Table
PRU_LOCAL_MEMORY				.SET	C24			;Points to PRU-ICSS PRU0/1 Data RAM (local), C24 = 0x0000_0n00, n = c24_blk_index[3:0]
CONST_ICSS_INTC					.SET	C0			;Points to PRU-ICSS INTC (local), C0 = 0x0002_0000

;From INTC
ICSS_INTC_SISR					.SET	0x0020		;Points to System Event Status Indexed Set Register for setting the status of an event
ICSS_INTC_SICR					.SET	0x0024		;Points to System Event Status Indexed Clear Register for clearing the status of an event

;For clearing System Event 16 and 56
CLEAR_EV_16_VAL					.SET	0x10
CLEAR_EV_56_VAL					.SET	0x38
CLEAR_EV_17_VAL					.SET	0x11
CLEAR_EV_18_VAL					.SET	0x12

;SYS_EVENT_16 is pr0_pru_mst_intr0_intr_req which generates a SYS_EVENT_56 for INTC of ICSS1
;ICSS1 then generates a system event for EDMA and EDMA then makes the change in pinmux for SDA
PRU_SYS_EVT_16					.SET	(16 - 16)
PRU0_R31_VEC_VALID				.SET	32
PR0_PR1_EDMA_INTR				.SET	(PRU0_R31_VEC_VALID | PRU_SYS_EVT_16)

PRU_SYS_EVT_17					.SET	(17 - 16)
PRU2ARM_INTR					.SET	(PRU0_R31_VEC_VALID | PRU_SYS_EVT_17)

PRU_SYS_EVT_18					.SET	(18 - 16)
ARM2PRU_INTR					.SET	(PRU0_R31_VEC_VALID | PRU_SYS_EVT_18)

;************************************************************************************
; 							Assembler Directives Section
;************************************************************************************
	.sect	".text: 0x00000000"
	.retain 										; Required for building .out with assembly file
	.retainrefs 									; Required for building .out with assembly file
	.sslist											; Required for building registers with names
	.global main

;==========================================		Main Program	====================================================
;	Function	:	Initializes the Communication Link, Executes either Testing Operation or SMBus Command in
;					Normal Operation and then waits for ARM Event for next operation to be executed
;-------------------------------------------------------------------------------------------------------------------
main:
	;Initialization of Communication Link (Sets Initial values for registers and Delay Values)
	M_CALL1		INIT_COMM_LINK
	M_CALL1		CHECK_AND_SET_PINMUX

	;Setting access ready flag for ARM so that it can read contents from register interface
	LBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4
	SET			status_value_IRQSTS_RAW, status_value_IRQSTS_RAW, ARDY_BIT
	SBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4

	;After executing either a testing operation or normal operation the PRU will wait here for an ARM event
WAIT_FOR_ARM_EVENT:
	QBBC		WAIT_FOR_ARM_EVENT, R31, PRU_ICSS0_EVENT_BIT

	;Clear interrupt flag from ARM
	M_CLR_INTC_ARM2PRU

	;Clear ARDY so that ARM cannot access the register interface
	LBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4
	CLR			status_value_IRQSTS_RAW, status_value_IRQSTS_RAW, ARDY_BIT
	SBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4

	;Checks which SMBus Command needs to be executed in Normal operation and then jumps to that Command's Label
NORMAL_OPERATION:
	LBCO		&SMBus_protocol_sel_reg, PRU_LOCAL_MEMORY, SELECTED_SMBUS_COMMAND, 1

	;SMBus Protocols
	QBEQ		QUICK_CMD_SELECTED, SMBus_protocol_sel_reg, QUICK_COMMAND
	QBEQ		SEND_BYTE_CMD_SELECTED, SMBus_protocol_sel_reg, SEND_BYTE
	QBEQ		RECEIVE_BYTE_CMD_SELECTED, SMBus_protocol_sel_reg, RECEIVE_BYTE
	QBEQ		WRITE_BYTE_CMD_SELECTED, SMBus_protocol_sel_reg, WRITE_BYTE
	QBEQ		WRITE_WORD_CMD_SELECTED, SMBus_protocol_sel_reg, WRITE_WORD
	QBEQ		READ_BYTE_CMD_SELECTED, SMBus_protocol_sel_reg, READ_BYTE
	QBEQ		READ_WORD_CMD_SELECTED, SMBus_protocol_sel_reg, READ_WORD
	QBEQ		WRITE_BLOCK_CMD_SELECTED, SMBus_protocol_sel_reg, WRITE_BLK
	QBEQ		READ_BLOCK_CMD_SELECTED, SMBus_protocol_sel_reg, READ_BLK

	;After executing normal operation the PRU will update the status registers to let the ARM know
	;what happened with the operation (Correct Operation, Invalid Command, No ACK from Slave etc)
SMBUS_TRANSACTION_COMPLETED:

	LBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4
	QBBS		CONTINUE_WITH_NO_ACK_CONDITION, ack_reg, SDA
	;Clearing NACK flag for ARM so that it knows there was acknowledge from slave
	CLR			status_value_IRQSTS_RAW, status_value_IRQSTS_RAW, NACK_BIT
	QBA			CONTINUE_WITH_ACK_CONDITION
CONTINUE_WITH_NO_ACK_CONDITION:
	;Setting NACK flag for ARM so that it knows there was no acknowledge from slave
	SET			status_value_IRQSTS_RAW, status_value_IRQSTS_RAW, NACK_BIT
CONTINUE_WITH_ACK_CONDITION:

	;Setting access ready flag for ARM so that it can read contents from register interface
	SET			status_value_IRQSTS_RAW, status_value_IRQSTS_RAW, ARDY_BIT
	SBCO		&status_value_IRQSTS_RAW, PRU_LOCAL_MEMORY, I2C_IRQSTS_RAW, 4

	;Checking the IRQEN_SET flag for generating ARM event or not
	LBCO		&status_value_IRQEN_SET, PRU_LOCAL_MEMORY, I2C_IRQEN_SET, 4
	AND			status_value, status_value_IRQSTS_RAW, status_value_IRQEN_SET
	QBEQ		WAIT_FOR_ARM_EVENT, status_value, 0
	;Generate event for ARM
	LDI			R31.b0, PRU2ARM_INTR
	;The PRU will jump above TEST_SELECTION Label and wait for ARM event after generating and interrupt for ARM
	QBA 		WAIT_FOR_ARM_EVENT

END:
	HALT
;=====================================================================================================================

;+++++++++++++++++++++++++++++++++++++++++++	SMBUS COMMANDS SECTION 	 +++++++++++++++++++++++++++++++++++++++++++++


;===============================================	Quick Command	==================================================
;
;					-------------------------------------
;	Format		:	| S | Slave Address | Rd/Wr | A | P |
;					-------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Rd/Wr
;					to the slave and checks Ack from slave and then finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
QUICK_CMD_SELECTED:

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Rd/Wr bit (Slave Address | Rd/Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	LBCO		&rd_wr_bit, PRU_LOCAL_MEMORY, I2C_CON, 1
	QBBS		TX_SELECTED, rd_wr_bit, READ_WRITE_BIT
	OR			tx_data, slave_addr, WRITE_BIT
	QBA			WRITE_DATA
TX_SELECTED:
	OR			tx_data, slave_addr, READ_BIT
WRITE_DATA:
	;Write the Slave Address (7 bits) & Rd/Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A)
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX9, ack_reg, SDA

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE9
IF_NO_ACK_TX9:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE9:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Send Byte Command	==================================================
;
;					---------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Data | A | P |
;					---------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr
;					to the slave and checks Ack from slave, then sends 8 bits of data and checks Ack from slave
;					and then finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
SEND_BYTE_CMD_SELECTED:
	LDI 		mem_idx_tx_data, TX_DATA_LOCATION

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX8, ack_reg, SDA

	;Load the Data to be transmitted to Slave (Data)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;LDI		tx_data, 0x30
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Data to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A)
	M_CALL1		CHECK_ACK_BIT_TX

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE8
IF_NO_ACK_TX8:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE8:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Receive Byte Command	==============================================
;
;					----------------------------------------------
;	Format		:	| S | Slave Address | Rd | A | Data' | N | P |
;					----------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Rd
;					from the slave and checks Ack from slave, then receives 8 bits of data from slave and then sends
;					Nack to slave and finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
RECEIVE_BYTE_CMD_SELECTED:
	LDI 		mem_idx_rx_data, RX_DATA_LOCATION

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Rd bit (Slave Address | Rd)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, READ_BIT
	;Write the Slave Address (7 bits) & Rd (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX3, ack_reg, SDA

	;Read the 8 bits of Data from slave (Data')
	M_CALL1		READ_BYTE_FROM_SLAVE
	;Store the Data Read from the Slave
	SBCO		&rx_data, PRU_LOCAL_MEMORY, mem_idx_rx_data, 1
	;Send NAck to slave (N) to terminate the communication
	M_CALL1		MASTER_SEND_NACK_TO_SLAVE

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE3
IF_NO_ACK_RX3:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION_RX
CONTINUE3:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Write Byte Command	==================================================
;
;					----------------------------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Command Code | A | Data | A | P |
;					----------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr
;					to the slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks
;					Ack from slave, then sends  8 bits of data and checks Ack from slave and then finally terminates
;					with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
WRITE_BYTE_CMD_SELECTED:
	LDI 		mem_idx_tx_data, TX_DATA_LOCATION

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX1, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI 		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX1, ack_reg, SDA

	;Load the Data to be transmitted to Slave (Data)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing data to be written checked with different values
	;LDI 		tx_data, 01010000b			;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x50				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x18				;for TUSB8041 serNumStringLen Register
	;LDI 		tx_data, 0x1F				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x50				;any value here that is sent actually
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Data to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A)
	M_CALL1		CHECK_ACK_BIT_TX

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE1
IF_NO_ACK_TX1:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE1:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Write Word Command	==================================================
;
;					------------------------------------------------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Command Code | A | Data Low | A | Data High | A | P |
;					------------------------------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr to the
;					slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks Ack from
;					slave, then sends 8 bits of low data byte and checks Ack from slave, then sends 8 bits of high
;					data byte and checks Ack from slave and then finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
WRITE_WORD_CMD_SELECTED:
	LDI 		mem_idx_tx_data, TX_DATA_LOCATION
	LDI 		byte_count, 0

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX4, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI 		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX4, ack_reg, SDA

	;Write data byte low (Data Low) and check Ack from Slave (A) then
	;write data byte high (Data High) and check Ack (A) from Slave
REPEAT_WRITE_LOOP:
	;Load the Data to be transmitted to Slave
	LBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;Write the 8 bits of Data to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave
	M_CALL1		CHECK_ACK_BIT_TX
	ADD			mem_idx_tx_data, mem_idx_tx_data, 1
	ADD			byte_count, byte_count, 1
	;If no Ack then generate stop bit
	QBBS		IF_NO_ACK_TX4, ack_reg, SDA
	;Loop until both bytes of data word are written to slave
	QBGT		REPEAT_WRITE_LOOP, byte_count, 2

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE4
IF_NO_ACK_TX4:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE4:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Read Byte Command	==================================================
;
;					----------------------------------------------------------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Command Code | A | Sr | Slave Address | Rd | A | Data' | N | P |
;					----------------------------------------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr to the
;					slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks Ack from
;					slave, then generates a Repeated Start bit , followed by same Slave Address appended with 1 bit
;					for Rd to the slave and checks Ack from slave, then receives 8 bits of data from slave and then
;					sends Nack to slave and finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
READ_BYTE_CMD_SELECTED:
	LDI 		mem_idx_rx_data, RX_DATA_LOCATION

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX2, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX2, ack_reg, SDA

	;Generate repeated start condition bit (Sr)
	M_CALL1		REPEATED_START_CONDITION

	;Read the Slave Address from the memory & append it with the Rd bit (Slave Address | Rd)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, READ_BIT
	;Write the Slave Address (7 bits) & Rd (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX2, ack_reg, SDA

	;Read the 8 bits of Data from slave (Data')
	M_CALL1		READ_BYTE_FROM_SLAVE
	;Store the Data Read from the Slave in memory
	SBCO		&rx_data, PRU_LOCAL_MEMORY, mem_idx_rx_data, 1
	;Send NAck to slave (N) to terminate the communication
	M_CALL1		MASTER_SEND_NACK_TO_SLAVE

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE2
IF_NO_ACK_RX2:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION_RX
	QBA			CONTINUE2
IF_NO_ACK_TX2:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE2:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Read Word Command	==================================================
;
;					--------------------------------------------------------------------------------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Command Code | A | Sr | Slave Address | Rd | A | Data Low' | A | Data High' | N | P |
;					--------------------------------------------------------------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr to the
;					slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks Ack from
;					slave, then generates a Repeated Start bit, followed by same Slave Address appended with 1 bit
;					for Rd to the slave and checks Ack from slave, then receives 8 bits of data low from slave and
;					sends Ack to slave, then receives 8 bits of data high and then sends Nack to slave and finally
;					terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
READ_WORD_CMD_SELECTED:
	LDI 		mem_idx_rx_data, RX_DATA_LOCATION
	LDI 		byte_count, 0

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX5, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI 		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX5, ack_reg, SDA

	;Generate repeated start condition bit (Sr)
	M_CALL1		REPEATED_START_CONDITION

	;Read the Slave Address from the memory & append it with the Rd bit (Slave Address | Rd)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, READ_BIT
	;Write the Slave Address (7 bits) & Rd (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX5, ack_reg, SDA

	;Read data byte low (Data Low') and send Ack to Slave (A) then
	;read data byte high (Data High') and send NAck (N) to Slave
REPEAT_READ_LOOP:
	;Read the 8 bits of Data from slave
	M_CALL1		READ_BYTE_FROM_SLAVE
	;Store the Data Read from the Slave in memory
	SBCO		&rx_data, PRU_LOCAL_MEMORY, mem_idx_rx_data, 1
	ADD			mem_idx_rx_data, mem_idx_rx_data, 1
	ADD 		byte_count, byte_count, 1
	;If both data bytes are received send NAck to Slave
	QBEQ		MASTER_ENDS_WITH_NACK, byte_count, 2
	;Send Ack to Slave
	M_CALL1		MASTER_SEND_ACK_TO_SLAVE
	;Loop until both bytes of data word are read from slave
	QBGT		REPEAT_READ_LOOP, byte_count, 2
MASTER_ENDS_WITH_NACK:
	;Send NAck (N) to slave to terminate the communication
	M_CALL1		MASTER_SEND_NACK_TO_SLAVE

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE5
IF_NO_ACK_RX5:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION_RX
	QBA			CONTINUE5
IF_NO_ACK_TX5:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE5:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Write Block Command	 =================================================
;
;					---------------------------------------------------------------------------------------------------------------------
;	Format		:	| S | Slave address | Wr | A | Command Code | A | Byte Count (N) | A | Data Byte 1 | A | .... | Data Byte N | A | P |
;					---------------------------------------------------------------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr to the
;					slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks Ack from
;					slave, then sends 8 bits of Byte Count (N) (tells slave how many data bytes will follow (Max 32))
;					and checks Ack from slave, then sends 8 bits of Data Byte 1 and check Ack from slave, then repeats
;					this step until Data Byte N and then finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
WRITE_BLOCK_CMD_SELECTED:
	LDI 		mem_idx_tx_data, TX_DATA_LOCATION
	LDI 		byte_count, 0
	LDI 		total_byte_count_N, 0
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;LDI 		tx_test_data, 0x0			;set initial data (incremented each iteration)
	;-----------------------------------------------------------------------------------

	;Generate Start bit (S)
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX6, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI 		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX6, ack_reg, SDA

	;Load the Byte Count (N) (Max 32) to be transmitted to Slave (Byte Count (N))
	LBCO		&tx_data, PRU_LOCAL_MEMORY, I2C_CNT, 1
	;If Byte Count > 32 then set Byte Count = 32 otherwise leave it as it is
	;QBGT		IF_TX_BYTE_COUNT_LESSER_THAN_32, tx_data, 32								;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
	;LDI		tx_data, 32																	;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing with different Byte Count (N) values
	;LDI		tx_data, 0x1F
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, I2C_CNT, 1
	;-----------------------------------------------------------------------------------
;IF_TX_BYTE_COUNT_LESSER_THAN_32:															;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
	ADD			total_byte_count_N, tx_data, 0
	;Write the 8 bits of Byte Count (N) to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX6, ack_reg, SDA

	;Write first data byte (Data Byte 1) and check Ack from Slave (A) then repeat (....)
	;until last data byte (Data Byte N) is written and check Ack (A) from Slave
WRITE_LOOP:
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;setting the Data to transmit in each iteration
	;ADD		tx_test_data, tx_test_data, 1
	;SBCO		&tx_test_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;-----------------------------------------------------------------------------------
	;Load the Data to be transmitted to Slave
	LBCO		&tx_data, PRU_LOCAL_MEMORY, mem_idx_tx_data, 1
	;Write the 8 bits of Data to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave
	M_CALL1		CHECK_ACK_BIT_TX
	ADD			mem_idx_tx_data, mem_idx_tx_data, 1
	ADD			byte_count, byte_count, 1
	;if no Ack then generate stop bit
	QBBS		IF_NO_ACK_TX6, ack_reg, SDA
	;Loop until N bytes of data word are written to slave
	QBGT		WRITE_LOOP, byte_count, total_byte_count_N

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE6
IF_NO_ACK_TX6:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE6:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;============================================	Read Block Command	 =================================================
;
;					------------------------------------------------------------------------------------------------------------------------------------------------------
;	Format		:	| S | Slave Address | Wr | A | Command Code | A | Sr | Slave Address | Rd | A | Byte Count (N)' | A | Data Byte 1' | A | .... | Data Byte N' | N | P |
;					------------------------------------------------------------------------------------------------------------------------------------------------------
;
;	Function	:	Generates Start bit, followed by writing 7 bits of Slave Address appended with 1 bit for Wr to the
;					slave and checks Ack from slave, then sends 8 bits of Command Code to slave and checks Ack from
;					slave, then generates a Repeated Start bit, followed by same Slave Address appended with 1 bit
;					for Rd to the slave and checks Ack from slave, then receives 8 bits of Byte Count (N)' from slave
;					(tells how many data bytes will follow (Max 32)) and sends Ack to slave, then receives 8 bits of
;					Data Byte 1' from slave and sends Ack to slave, then repeats this step until Data Byte N and
;					finally terminates with generating a Stop bit
;---------------------------------------------------------------------------------------------------------------------
READ_BLOCK_CMD_SELECTED:
	LDI 		mem_idx_rx_data, RX_DATA_LOCATION
	LDI 		byte_count, 0
	LDI 		rx_total_byte_count_N, 0

	; Generate Start bit
	M_CALL1		START_BIT_GENERATION

	;Read the Slave Address from the memory & append it with the Wr bit (Slave Address | Wr)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, WRITE_BIT
	;Write the Slave Address (7 bits) & Wr (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_TX
	QBBS		IF_NO_ACK_TX7, ack_reg, SDA

	;Load the Command Code to be transmitted to Slave (Command Code)
	LBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing command code written with different values
	;LDI 		tx_data, 0x5				;for TUSB8041 Configuration Register
	;LDI 		tx_data, 0x22				;for TUSB8041 serNumStringLen Register containing
											;Bytes to be read from serialNumber Register
	;LDI 		tx_data, 0x30				;for TUSB8041 serialNumber Register
	;LDI 		tx_data, 0xF8				;for TUSB8041 Device Status and Command Register
	;LDI 		tx_data, 0x5				;any value here for checking other Registers
	;SBCO		&tx_data, PRU_LOCAL_MEMORY, COMMAND_CODE, 1
	;-----------------------------------------------------------------------------------
	;Write the 8 bits of Command Code to Slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX7, ack_reg, SDA

	;Generate repeated start condition bit (Sr)
	M_CALL1		REPEATED_START_CONDITION

	;Read the Slave Address from the memory & append it with the Rd bit (Slave Address | Rd)
	LBCO		&slave_addr, PRU_LOCAL_MEMORY, I2C_SA, 1
	LSL			slave_addr, slave_addr, 1
	OR			tx_data, slave_addr, READ_BIT
	;Write the Slave Address (7 bits) & Rd (1 bit) to slave
	M_CALL1		WRITE_ONE_BYTE_TO_SLAVE
	;Check Ack from Slave (A), if no Ack then generate stop bit
	M_CALL1		CHECK_ACK_BIT_RX
	QBBS		IF_NO_ACK_RX7, ack_reg, SDA

	;Read the 8 bits of Byte Count (N)' from Slave
	M_CALL1		READ_BYTE_FROM_SLAVE
	;--------------------- This Code is for Testing & Validation Purpose ---------------
	;for testing with different Byte Count (N)' values
	;LDI rx_data, 0x3
	;-----------------------------------------------------------------------------------
	;Store the Byte Count Read from the Slave in memory
	SBCO		&rx_data, PRU_LOCAL_MEMORY, I2C_CNT, 1
	;If Byte Count' > 32 then set Byte Count' = 32 otherwise leave it as it is
	;QBGT		IF_RX_BYTE_COUNT_LESSER_THAN_32, rx_data, 32								;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
	;LDI			rx_data, 32																;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
;IF_RX_BYTE_COUNT_LESSER_THAN_32:															;Uncomment for SMBus 2.0 Specification which allows max 32 bytes unlike 256 bytes in 3.0
	ADD			rx_total_byte_count_N, rx_data, 0
	;Send Ack (A) to Slave
	M_CALL1		MASTER_SEND_ACK_TO_SLAVE

	;Read first data byte (Data Byte 1') and send Ack to Slave (A) then repeat (....)
	;until last data byte (Data Byte N) is read and send NAck (N) to Slave
READ_LOOP:
	;Read the 8 bits of Data from slave
	M_CALL1		READ_BYTE_FROM_SLAVE
	;Store the Data Read from the Slave in memory
	SBCO		&rx_data, PRU_LOCAL_MEMORY, mem_idx_rx_data, 1
	ADD			mem_idx_rx_data, mem_idx_rx_data, 1
	ADD			byte_count, byte_count, 1
	;If N bytes of data are received send NAck to Slave
	QBEQ		MASTER_ENDS_BLOCK_READ_WITH_NACK, byte_count, rx_total_byte_count_N
	;Send Ack to Slave
	M_CALL1		MASTER_SEND_ACK_TO_SLAVE
	;Loop until N bytes of data word are read from slave
	QBGT		READ_LOOP, byte_count, rx_total_byte_count_N
MASTER_ENDS_BLOCK_READ_WITH_NACK:
	;Send NAck (N) to slave to terminate the communication
	M_CALL1		MASTER_SEND_NACK_TO_SLAVE

	;Generate Stop bit (P)
	M_CALL1		STOP_BIT_GENERATION

	QBA			CONTINUE7
IF_NO_ACK_RX7:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION_RX
	QBA			CONTINUE7
IF_NO_ACK_TX7:
	;Generate Stop bit in case of no Ack from slave
	M_CALL1		STOP_BIT_GENERATION
CONTINUE7:
	;To go back in main program
	QBA			SMBUS_TRANSACTION_COMPLETED
;=====================================================================================================================

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;++++++++++++++++++++++++++++++++++   BUILDING BLOCKS OF SMBUS COMMANDS SECTION   ++++++++++++++++++++++++++++++++++++


;=============================================    INIT_COMM_LINK   ===================================================
;
;	Function	:	-Initializes all the registers
;					-Loads the frequency and quater clock cycle delay value from memory
;					-Sets half clock cycle delay value
;					-Initializes the delay registers with the corresponding values
;---------------------------------------------------------------------------------------------------------------------
INIT_COMM_LINK:
	;Clear all registers
	ZERO		&R0, 112

	;Load half clock cycle delay value from memory
	LBCO		&delay_count_reg_half_cycle, PRU_LOCAL_MEMORY, CLOCK_HALF_CYCLE_DELAY_VALUE, 2
	;Set quater clock cycle delay value (half clock cycle delay value/2)
	LSR			delay_count_reg_quater_cycle, delay_count_reg_half_cycle, 1

	;Clear the count registers
	LDI 		bit_count, 0
	LDI			total_byte_count_N, 0
	LDI			rx_total_byte_count_N, 0
	;Initialize the required registers
	LDI			mem_idx_tx_data, TX_DATA_LOCATION
	LDI			mem_idx_rx_data, RX_DATA_LOCATION

	;Load the address for PRU1-ICSS1 INTC
	M_MOV32		pru1_intc, 0x54420000 		;0x40000 + 0x20000 (To access locally) ;0x54420000 (To access globally)
	;Load the address of Control Module (0x44E10000) + Offset for CTRL_CONF_CAM1_DATA3 (0x9F4) the pin used as SDA
	M_MOV32		sda_pin_reg, 0x44E109F4
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	;Return to the calling body
	M_RET1
;=====================================================================================================================

;==========================================    CHECK_AND_SET_PINMUX   ================================================
;
;	Function			:	-Checks the initial value of the pinmux register for SDA pin
;							-Makes sure SDA is input upon exit to enable Pull-up on the Master side
;
;	SDA Pin on Entry	:	Input / Output
;	SDA Pin inside body	:	Input / Output -> Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
CHECK_AND_SET_PINMUX:

	;Read the pinmux value of the SDA
	LBBO		&pnmx_val, sda_pin_reg, 0, 1
	;If it is output (4), Set it to input (5)
	QBEQ		SET_CLK, pnmx_val, 5

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux value changed)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

SET_CLK:
	SET			R30, R30, SCL
	M_CALL		DELAY_HALF_CYCLE
	;Return to the calling body
	M_RET1
;=====================================================================================================================

;==========================================    START_BIT_GENERATION   ================================================
;
;	Function			:	-Generates Start bit condition (S) for I2C/SMBus transaction
;							-Switches SDA from Input to Output
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input -> Output
;	SDA Pin on Exit		:	Output
;---------------------------------------------------------------------------------------------------------------------
START_BIT_GENERATION:
	SET 		R30, R30, SDA
	;Start clock
	M_CALL		DELAY_HALF_CYCLE
	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
	;Pull the data line (SDL) while clock (SCL) is high to indicate start bit
	CLR 		R30, R30, SDA
	M_CALL		DELAY_HALF_CYCLE
	;Return to the calling body
	M_RET1
;=====================================================================================================================

;=======================================    REPEATED_START_CONDITION    ==============================================
;
;	Function			:	-Generates Repeated Start bit condition (Sr) for I2C/SMBus transaction
;							-Switches SDA from Input to Output
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input -> Output
;	SDA Pin on Exit		:	Output
;---------------------------------------------------------------------------------------------------------------------
REPEATED_START_CONDITION:
	CLR			R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE
	SET			R30, R30, SDA
	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	SET			R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE
	;Pull the data line (SDA) while clock (SCL) is high to indicate Repeated Start bit condition (Sr)
	CLR 		R30, R30, SDA
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	M_CALL		DELAY_QUATER_CYCLE
	;Return to the calling body
	M_RET1
;=====================================================================================================================

;========================================    WRITE_ONE_BYTE_TO_SLAVE    ==============================================
;
;	Function			:	-Writes one byte to the slave device which can be Slave Address | Rd/Wr, Command Code,
;							 Byte Count (N) or Normal Data byte
;
;	SDA Pin on Entry	:	Output
;	SDA Pin inside body	:	Output / Output -> Input
;	SDA Pin on Exit		:	Output / Input
;---------------------------------------------------------------------------------------------------------------------
WRITE_ONE_BYTE_TO_SLAVE:

	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Sends the MSB in each clock cycle then shifts one bit to the left until all 8 bits are sent this way
	LDI 		bit_count, 0
	LDI			last_bit_sent, 0x0
	LDI			bit_count_and_last_bit_sent, 0x0
START_TX:
	CLR			R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE
	;If MSB of tx_data is '1' then write 1 to SDA otherwise 0
	QBBS		SET_SDA, tx_data, 7
	LDI			last_bit_sent, 0x0
	CLR			R30, R30, SDA
	QBA			SHIFT_DATA
SET_SDA:
	LDI			last_bit_sent, 0x1
	SET			R30, R30, SDA
SHIFT_DATA:
	LSL 		tx_data, tx_data, 1						;tx_data <<= 1
	ADD 		bit_count, bit_count, 1					;var bit_count += 1
	;If last bit of tx_data is '1' then trigger EDMA for pinmux earlier at mid-way of negative clock period
	;otherwise normally at the falling edge of clock from function CHECK_ACK_BIT_TX or CHECK_ACK_BIT_RX
	ADD			bit_count_and_last_bit_sent, bit_count, last_bit_sent
	QBGT		IF_LAST_BIT_SENT_NOT_1, bit_count_and_last_bit_sent, 9
	LDI			R31.b0, PR0_PR1_EDMA_INTR
IF_LAST_BIT_SENT_NOT_1:
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_HALF_CYCLE
	;Loop until all 8 bits are sent
	QBGT 		START_TX , bit_count, 8					;check bit_count < 8

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;==========================================    READ_BYTE_FROM_SLAVE    ===============================================
;
;	Function			:	-Reads one byte from the slave device which can be Byte Count (N)' or Normal Data' byte
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
READ_BYTE_FROM_SLAVE:

	;Sets the LSB in each clock cycle according to SDA then shifts one bit to the left until all 8 bits are attained this way
	CLR    		R30, R30, SCL
	M_CALL 		DELAY_HALF_CYCLE
	LSL    		rx_data, rx_data, 1						; rx_data <<= 1
	SET   		R30, R30, SCL
	M_CALL 		DELAY_QUATER_CYCLE
	;If SDA is '1' write 1 to LSB of rx_data otherwise 0
	QBBS   		RECEIVED_ONE, R31, SDA
	CLR    		rx_data, rx_data, 0
	QBA    		CLR_CLK_RX
RECEIVED_ONE:
	SET    		rx_data, rx_data, 0
CLR_CLK_RX:
	M_CALL  	DELAY_QUATER_CYCLE
	ADD    		bit_count, bit_count, 1					;var bit_count += 1
	;Loop until all 8 bits are received
	QBGT   		READ_BYTE_FROM_SLAVE , bit_count, 8		;check bit_count < 8
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;============================================    CHECK_ACK_BIT_TX    =================================================
;
;	Function			:	-Switches SDA from Output to Input
;							-Checks for ACK bit during read operation
;							-Finally switches SDA from Input back to Output
;
;	SDA Pin on Entry	:	Output	/ Input
;	SDA Pin inside body	:	Output -> Input -> Output / Input -> Output
;	SDA Pin on Exit		:	Output
;---------------------------------------------------------------------------------------------------------------------
CHECK_ACK_BIT_TX:
	LDI 		bit_count, 0
	CLR 		R30, R30, SCL
	CLR 		R30, R30, SDA

	;If last bit of tx_data is '1' then trigger EDMA for pinmux earlier from function WRITE_ONE_BYTE_TO_SLAVE at mid-way
	;of negative clock period otherwise normally at the falling edge of clock from function CHECK_ACK_BIT_TX
	QBEQ		IF_LAST_BIT_SENT_1, last_bit_sent, 0x1

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Output to Input)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
IF_LAST_BIT_SENT_1:
	M_CALL		DELAY_HALF_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE

	;Read 11th bit of R31 - SDA line has to be pulled down by the slave indicating the ACK bit
	MOV			ack_reg, R31.w0
	;Internal SDA value follows the value from slave to avoid multiple driver glitch during pinmux change
	QBBS		SET_SDA_1, ack_reg, SDA
	;If Ack = 0 set SDA = 0 to avoid multiple driver glitch
	CLR 		R30, R30, SDA
	QBA 		SAMPLE_TAKEN
SET_SDA_1:
	;If Ack = 1 set SDA = 1 to avoid multiple driver glitch
	SET 		R30, R30, SDA
SAMPLE_TAKEN:
	M_CALL	DELAY_QUATER_CYCLE

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
	;M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;============================================    CHECK_ACK_BIT_RX    =================================================
;
;	Function			:	-Switches SDA from Output to Input
;							-Checks for ACK bit during read operation
;
;	SDA Pin on Entry	:	Output
;	SDA Pin inside body	:	Output -> Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
CHECK_ACK_BIT_RX:
	LDI 		bit_count, 0
	CLR 		R30, R30, SCL
	CLR 		R30, R30, SDA
	;M_CALL		DELAY_QUATER_CYCLE

	;If last bit of tx_data is '1' then trigger EDMA for pinmux earlier from function WRITE_ONE_BYTE_TO_SLAVE at mid-way
	;of negative clock period otherwise normally at the falling edge of clock from function CHECK_ACK_BIT_RX
	QBEQ		IF_LAST_BIT_SENT_IS_1, last_bit_sent, 0x1

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Output to Input)
	LDI			R31.b0, PR0_PR1_EDMA_INTR
IF_LAST_BIT_SENT_IS_1:
	M_CALL		DELAY_HALF_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE

	;Read 11th bit of R31 - SDA line has to be pulled down by the slave indicating the ACK bit
	MOV			ack_reg, R31.w0
	;Internal SDA value follows the value from slave to avoid multiple driver glitch during pinmux change
	QBBS		Label_SET_SDA_1, ack_reg, SDA
	;If Ack = 0 set SDA = 0 to avoid multiple driver glitch
	CLR 		R30, R30, SDA
	QBA 		Label_SAMPLE_TAKEN
Label_SET_SDA_1:
	;If Ack = 1 set SDA = 1 to avoid multiple driver glitch
	SET 		R30, R30, SDA
Label_SAMPLE_TAKEN:
	M_CALL		DELAY_QUATER_CYCLE

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;========================================    MASTER_SEND_ACK_TO_SLAVE    =============================================
;
;	Function			:	-Switches SDA from Input to Output
;							-Generate the ACK bit to acknolwedge data reception from the slave
;							-Finally switches SDA from Output back to Input
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input -> Output -> Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
MASTER_SEND_ACK_TO_SLAVE:
	LDI    		bit_count, 0
	CLR 		R30, R30, SCL

	;Clear SDA - SDA line has to be pulled down by the master indicating the ACK bit
	CLR    		R30, R30, SDA

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	;so that slave can see SDA line pulled down by master signifying and ACK
	LDI    		R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	M_CALL  	DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_HALF_CYCLE

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Output to Input)
	LDI    		R31.b0, PR0_PR1_EDMA_INTR

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;========================================    MASTER_SEND_NACK_TO_SLAVE    ============================================
;
;	Function			:	-Switches SDA from Input to Output
;							-Generate the NACK bit to acknolwedge data reception from the slave and
;							 indicate end of Read Operation from the slave
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input -> Output
;	SDA Pin on Exit		:	Output
;---------------------------------------------------------------------------------------------------------------------
MASTER_SEND_NACK_TO_SLAVE:
	LDI    		bit_count, 0
	CLR 		R30, R30, SCL
	CLR		   	R30, R30, SDA

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	;So master can send NACK to end Read Operation
	LDI    		R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	SET    		R30, R30, SDA
	M_CALL  	DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_HALF_CYCLE

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;===========================================    STOP_BIT_GENERATION    ===============================================
;
;	Function			:	-Generates Stop bit condition (P) for I2C/SMBus transaction when SDA is Output
;							-Switches SDA from Output to Input
;
;	SDA Pin on Entry	:	Output
;	SDA Pin inside body	:	Output -> Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
STOP_BIT_GENERATION:

	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	CLR 		R30, R30, SCL

	;The following command moved from here and after quater cycle delay to generate the proper NACK from Master
	;CLR 		R30, R30, SDA
	M_CALL		DELAY_QUATER_CYCLE
	CLR 		R30, R30, SDA
	M_CALL		DELAY_QUATER_CYCLE
	SET 		R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Pullup the data line (SDA) while clock (SCL) is high to indicate Stop bit condition (P)
	SET 		R30, R30, SDA
	M_CALL		DELAY_QUATER_CYCLE

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Output to Input)
	LDI 		R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;==========================================    STOP_BIT_GENERATION_RX    =============================================
;
;	Function			:	-Generates Stop bit condition (P) for I2C/SMBus transaction when SDA is Input
;							STEPS:
;							-Switches SDA from Input to Output
;							-Generates Stop bit
;							-Switches SDA from Output back to Output
;
;	SDA Pin on Entry	:	Input
;	SDA Pin inside body	:	Input -> Output -> Input
;	SDA Pin on Exit		:	Input
;---------------------------------------------------------------------------------------------------------------------
STOP_BIT_GENERATION_RX:

	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	CLR 		R30, R30, SCL
	CLR 		R30, R30, SDA
	M_CALL		DELAY_QUATER_CYCLE

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Input to Output)
	LDI 		R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX
	SET 		R30, R30, SCL
	M_CALL		DELAY_QUATER_CYCLE

	;Pullup the data line (SDA) while clock (SCL) is high to indicate Stop bit condition (P)
	SET 		R30, R30, SDA
	M_CALL		DELAY_QUATER_CYCLE

	;Trigger pinmux change (Generate event 16 ICSS0-INTC -> Generate event 56 ICSS1-INTC -> EDMA -> Pinmux Output to Input)
	LDI 		R31.b0, PR0_PR1_EDMA_INTR
	M_CALL		DELAY_QUATER_CYCLE
	;Clear the interrupt flags of ICSS0 INTC and ICSS1 INTC
	M_CLR_INTC_PINMUX

	;Return to the calling body
	M_RET1
;=====================================================================================================================

;============================================    DELAY_QUATER_CYCLE    ===============================================
;
;	Function			:	-Generates the delay required for producing quater clock cycle
;							-delay value ~= z/4 instructions
;
;	* PRU takes 5ns to execute one instruction, for y ns it should execute, z = (yns/5ns) instructions
;	  - Ex : For 100KHz clock, 1/100KHz = 10us, So for y = 10us, z = 10u/5n = 2000 instructions
;	  - To set the clock it takes z/2 number of instructions and to clear the clock it takes z/2 number of instructions
;
;---------------------------------------------------------------------------------------------------------------------
DELAY_QUATER_CYCLE:
	;do nothing for z/4 instructions
	loop 		NO_OP, delay_count_reg_quater_cycle		;delay_count_reg_quater_cycle ~= z/4 instructions
	M_NOP
NO_OP:
	;Return to the calling body
	M_RET
;=====================================================================================================================

;=============================================    DELAY_HALF_CYCLE    ================================================
;
;	Function			:	-Generates the delay required for producing quater clock cycle
;							-delay value ~= z/2 instructions
;
;	* PRU takes 5ns to execute one instruction, for y ns it should execute, z = (yns/5ns) instructions
;	  - Ex : For 100KHz clock, 1/100KHz = 10us, So for y = 10us, z = 10u/5n = 2000 instructions
;	  - To set the clock it takes z/2 number of instructions and to clear the clock it takes z/2 number of instructions
;
;---------------------------------------------------------------------------------------------------------------------
DELAY_HALF_CYCLE:
	;do nothing for z/2 instructions
	loop 		NO_OP_1, delay_count_reg_half_cycle		;delay_count_reg_half_cycle ~= z/2 instructions
	M_NOP
NO_OP_1:
	;Return to the calling body
	M_RET
;=====================================================================================================================

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
